@@ -1,7 +1,7 @@
 from .client import Client  # noqa
-from .common import Direction, PoolAsset, Side, TxConfig  # noqa
+from .common import Direction, PoolAsset, Side, TxConfig, Coin # noqa
 from .composer import Composer  # noqa
 from .network import Network  # noqa
 from .sdk import Sdk  # noqa
 from .transaction import Transaction  # noqa
-from .wallet import Address, PrivateKey, PublicKey  # noqa
+from .wallet import Address, PrivateKey, PublicKey  # noqa
\ No newline at end of file
